### Example code to assess temporal avoidance/attraction ("temporal spacing")
### between two species (A and B) from camera trap data  

### Author: Jeremy J. Cusack (jeremy.cusack@umayor.cl)

##### Set working directory #####

setwd("D:\\Onderzoek\\Aios & Postdocs\\Sander\\Data Chapter 3\\")

##### Packages #####

require(overlap)
require(adehabitatLT)

##### Custom functions #####

# Observed time to event function. Calculates the time elapsed between the detection of species A and the next detection of
# species B at the same camera trap station. If species A is re-detected before species B, the "time count" is reset

obs.time.to.event <- function(events.dom, events.sub, tunit = "hours"){
  Dom.name <- as.character(unique(events.dom$Species)) # Name of the dominant species
  Sub.name <- as.character(unique(events.sub$Species)) # Name of the subordinate species
  
  co.cams <- intersect(unique(events.dom$Site), unique(events.sub$Site)) # Camera stations at which the species co-occur
  
  delay.post <- vector() # Storage vector for time to events post detection of the dominant species
  delay.pre <- vector() # Storage vector for time to events pre detection of the dominant species
  
  for (s in co.cams){ # For each camera at which the species co-occur
    dom <- events.dom[events.dom$Site == s,] # Extract detections of the dominant species
    sub <- events.sub[events.sub$Site == s,] # Extract detections of the subordinate species
    x <- rbind(dom,sub) # Bind the two sets of events together
    chro <- x[order(x$DateTime),] # Order chronologically
    
    for (i in 1:(nrow(chro)-1)){ # For each row in chro (save the last)
      if (chro[i,"Species"] == Dom.name & chro[i+1,"Species"] == Sub.name){ # If the current event is of the dominant species and the next event of the subordinate species
        tmp <- as.numeric(difftime(chro[i+1,"DateTime"], chro[i,"DateTime"], units = tunit)) # Calculate time difference
        delay.post <- c(delay.post,tmp) # Add to the post storage vector
      }
      else{ # Otherwise
        delay.post <- c(delay.post, NA) # Insert NA
      }
    }
    for (i in 2:nrow(chro)){ # For each row in chro (save the first)
      if (chro[i,"Species"] == Dom.name & chro[i-1,"Species"] == Sub.name){ # If the current event is of the dominant species and the previous event of the subordinate species
        tmp <- as.numeric(difftime(chro[i-1,"DateTime"], chro[i,"DateTime"], units = tunit)) # Calculate time difference
        delay.pre <- c(delay.pre,tmp) # Add to the pre storage vector
      }
      else{ # Otherwise
        delay.pre <- c(delay.pre, NA) # Insert NA
      }
    }
  }
  c(delay.pre,delay.post) # Bind pre and post vectors together
}

# Function to randomise timestamps for one species. Resamples dates and times separately and then joins together. Ensures
# new timestamps are not outside of the species' activity pattern, nor outside of the period of camera activity. Note that this
# makes this randomisation procedure quite conservative.

rand.ct.times <- function(obs){
  
  date <- format(obs$DateTime,"%Y-%m-%d") # Extract dates
  time <- format(obs$DateTime,"%H:%M:%S") # Extract times
  
  samp.date <- sample(date,nrow(obs),replace=T) # resample dates with replacement
  samp.time <- sample(time,nrow(obs),replace=T) # resample times with replacement
  
  new.date.time <- strptime(paste(samp.date,samp.time),format="%Y-%m-%d %H:%M:%S") # join together as new date time
  obs$DateTime <- new.date.time # replace in dataset
  
  return(obs) # return randomised dataset
}

# Alternative function to randomise timestamps for one species. Resamples dates and times separately and then joins together. 
# Samples time from estimated species activity pattern.

rand.ct.times.with.act <- function(obs){
  
  ### Estimate activity pattern
  hr <- as.numeric(format(obs$DateTime, "%H")) # Extract event hour
  mn <- as.numeric(format(obs$DateTime, "%M")) # Extract event minute
  sc <- as.numeric(format(obs$DateTime, "%S")) # Extract event second
  dtime <- (3600*hr+60*mn+sc)/(24*3600) # Turn into decimal time
  rtime <- dtime*2*pi # Turn into radians
  bw <- getBandWidth(rtime) # Get the estimated bandwidth for the pdf
  tt <- seq(0,2*pi,length=60*24) # Create a minute grid
  dfit <- densityFit(rtime, grid=tt, bw=bw) # Estimate the density function for the activity pattern
  
  date <- format(obs$DateTime,"%Y-%m-%d") # Extract dates
  
  samp.date <- sample(date,nrow(obs),replace=T) # resample dates with replacement
  samp.time <- sample((tt/(2*pi))*(24*60),nrow(obs),prob=dfit,replace=T) # resample times with replacement
  
  new.date.time <- strptime(samp.date,format="%Y-%m-%d") # join together as new date time
  obs$DateTime <- new.date.time+samp.time*60 # replace in dataset
  
  return(obs) # return randomised dataset
}

##### Import example dataset #####

# Dataset with 3 columns: 
# Species - name of the photographed species
# Site - unique name of the camera station
# DateTime - combined date and time

#data <- read.csv("example_data.csv") 
data <- read.csv("leopard_brownhyena_wet_season2018tm2019tm2020.csv") 
data$DateTime <- strptime(data$DateTime, format="%d/%m/%Y %H:%M") # Format date and time
head(data)

##### Create species specific datasets (note species A and B can be switched around) #####

#spp.a <- data[data$Species == "Lion",] # Species A
#spp.b <- data[data$Species == "Spotted hyena",] # Species B
spp.a <- data[data$Species == "Leopard",] # Species A
spp.b <- data[data$Species == "Brown Hyena",] # Species B

##### Fix analysis parameters #####

reps=1000 # Number of randomisations

intvl=48 # Number of hours before and after detection of the dominant species

time.unit <- 6 # Unit used to divide intvl. Must be a multiple of intvl.

# NOTE - if your dataset only contains nighttime events and assuming one night = 12 hours, then it might make sense to set
# intvl to 12 and time.unit to 1. You can play around with this depending on how much data you have.

##### Example analysis #####

y <- obs.time.to.event(spp.a,spp.b) # Observed time to event
y <- y[-which(is.na(y)==T)] # Remove NAs
y <- y[which(y>=-intvl & y<=intvl)] # Select those within the interval of interest

# Derive observed detection probability for each specified time unit within the specified interval (also creates a plot)

OBS <- hist(y,breaks=seq(-intvl,intvl,time.unit),plot=F)$counts/nrow(spp.b) # Species A (lion in this case) detected at 0 hours

### RANDOMISATION
# NOTE: the randomisation procedure below can take quite some time to run. If trying for the first time it may be best to reduce 
# the number of randomisations.

EXP <- matrix(NA,nrow=reps,ncol=(intvl/time.unit*2)) # Storage matrix to store results from randomisations (no. of columns=days 
# in interval, no. rows=number of randomisations)
for (r in 1:reps){ # For each randomisation
  nw <- rand.ct.times(obs=spp.b) # Randomise observed event date and times
  nwy <- obs.time.to.event(spp.a, nw, tunit = "hours") # Derive time to event for randomised set of events
  nwy <- nwy[-which(is.na(nwy)==T)] # Remove NAs
  nwy <- nwy[which(nwy>=-intvl & nwy<=intvl)] # Select those within the interval of interest
  EXP[r,] <- hist(nwy,breaks=seq(-intvl,intvl,time.unit),plot=F)$counts/nrow(spp.b) # Store expected detection probabilities
  print(r) # Print randomisation number
}

##### Plot of null distribution, observed detection probability and p value per time division #####

par(mfrow=c(1,2))

time.unit.names <- c(seq(-intvl,-time.unit,time.unit),seq(time.unit,intvl,time.unit)) # Names of the time units for plottting purposes
colnames(EXP) <- time.unit.names

boxplot(EXP,xlab="Hours",col="grey90",ylab="Detection probability",outline=F,ylim=c(0,max(OBS))) # Plot null distributions per time unit. Note that the middle 
# of this plot will be 0, i.e. the detection of species A
points(OBS~c(1:ncol(EXP)),pch=16,col="red",cex=1.5) # Plot observed detection probabilities for species B

p.values <- vector() # Creat p value vector

for (c in 1:ncol(EXP)){
  
  p.values[c] <- as.randtest(sim=EXP[,c],obs=OBS[c],alter="two-sided")$pvalue
  
} # Derive p.value for each time division

barplot(p.values,names.arg=time.unit.names,ylim=c(0,1),col="grey90") # Plot p values
abline(h=0.05,lwd=2,col="blue") # Add alpha level of significance, here 0.05 (for example)

